@@ -1,9 +1,6 @@
 cc="`head -1 ../conf-cc`"
 systype="`cat systype`"
 
-ccqlibs="`head -1 ../conf-qlibs`"
-[ -d "$ccqlibs"/include ] && ccqlibs="-I${ccqlibs}/include" \
-|| ccqlibs=""
 
 cc -c trycpp.c -malign-double >/dev/null 2>&1 \
 && ccad="-malign-double"
@@ -23,10 +20,14 @@ cc -c trycpp.c -march=native >/dev/null 2>&1 \
 rm -f trycpp.o
 
 ccssl="`head -1 ../conf-ssl`"
-eval cc -c tryssl.c ${ccssl} >/dev/null 2>&1 \
+eval cc -c tryssl.c $ccssl >/dev/null 2>&1 \
 || ccssl=""
 
-ccbase="cc -fomit-frame-pointer -Wall"
+ccedh="`head -1 ../conf-ecdh`"
+eval cc -pedantic tryssl.c $ccedh >/dev/null 2>&1 \
+|| ccedh=""
+
+ccbase="gcc -fomit-frame-pointer -Wimplicit -Wunused -Wcomment -Wchar-subscripts -Wuninitialized -Wshadow -Wcast-qual -Wcast-align -Wwrite-strings"
 
 case "$cc:$systype" in
   auto:*:i386-*:*)
@@ -59,4 +60,4 @@ case "$cc:$systype" in
 esac
 
 cat warn-auto.sh
-echo exec "$cc" ${ccqlibs} ${ccssl} '-c ${1+"$@"}'
+echo exec "$cc" "$ccssl" "$ccedh" '-c ${1+"$@"}'
