@@ -14,7 +14,7 @@ int ssl_chainfile(SSL_CTX *ctx,const char *certchainfile,const char *keyfile,pem
     return -1;
 
   SSL_CTX_set_default_passwd_cb(ctx,passwd_cb);
-  if (SSL_CTX_use_PrivateKey_file(ctx,keyfile,SSL_FILETYPE_PEM) != 1)
+  if (SSL_CTX_use_RSAPrivateKey_file(ctx,keyfile,SSL_FILETYPE_PEM) != 1)
     return -2;
 
   if (SSL_CTX_check_private_key(ctx) != 1)
