@@ -6,6 +6,7 @@
 #include "ucspissl.h"
 #include "case.h"
 #include "strerr.h"
+#include "str.h"
 #include <openssl/x509v3.h>
 
 int ssl_verify(SSL *ssl,const char *hostname)
@@ -32,10 +33,14 @@ int ssl_verify(SSL *ssl,const char *hostname)
     for (i = 0; i < num; ++i) {
       ext = sk_GENERAL_NAME_value(extensions,i);
       if (ext->type == GEN_DNS) {
+//#if OPENSSL_VERSION_NUMBER < 0x10100000L
         if (ASN1_STRING_type(ext->d.ia5) != V_ASN1_IA5STRING) continue;
+//#else 
+        if (OBJ_sn2nid(ext->d.ia5) != V_ASN1_IA5STRING) continue;
+//#endif
         dnsname = (char *)ASN1_STRING_data(ext->d.ia5);
         len = ASN1_STRING_length(ext->d.ia5);
-        if (len != strlen(dnsname)) continue;
+        if (len != str_len(dnsname)) continue;
         if (case_diffs(hostname,dnsname) == 0) return 0;
         dname = 1;
       }
