@@ -109,7 +109,7 @@ const char *cadir = auto_cadir;
 const char *ciphers = auto_ciphers;
 int verifydepth = 1;
 const char *dhfile = auto_dhfile;
-int rsalen = 1024;
+int rsalen = SSL_RSA_LEN;
 
 char * const *prog;
 
@@ -195,19 +195,20 @@ void found(char *data,unsigned int datalen) {
 
 void doit(int t) {
   int j;
-  SSL *ssl;
+  SSL *ssl = 0;
   int wstat;
   int sslctl[2];
   char *s;
   unsigned long tmp_long;
   char ssl_cmd;
-  stralloc ssl_env = { 0 };
+  stralloc ssl_env = {0};
   int bytesleft;
   char envbuf[8192];
   int childpid;
   int mappedv4 = 0;
   uint32 scope_id;
   char *stripaddr;
+  stralloc tlsinfo = {0};
   
   if (pipe(pi) == -1) strerr_die2sys(111,DROP,"unable to create pipe: ");
   if (pipe(po) == -1) strerr_die2sys(111,DROP,"unable to create pipe: ");
@@ -225,25 +226,6 @@ void doit(int t) {
   else				/* IPv4 address */
     remoteipstr[ip6_compactaddr(remoteipstr,remoteip)] = 0;
      
-  if (flagremotehost)
-    if (dns_name6(&remotehostsa,remoteip) == 0)
-    if (remotehostsa.len) {
-      if (flagparanoid) {
-        verifyhost = remoteipstr;
-        if (dns_ip6(&tmp,&remotehostsa) == 0)
-          for (j = 0;j + 16 <= tmp.len;j += 16)
-            if (byte_equal(remoteip,16,tmp.s + j)) {
-              flagparanoid = 0;
-              break;
-          }
-      }
-      if (!flagparanoid) {
-        if (!stralloc_0(&remotehostsa)) drop_nomem();
-          remotehost = remotehostsa.s;
-          verifyhost = remotehostsa.s;
-      }
-    }
-    
   switch(childpid=fork()) {
     case -1:
       strerr_die2sys(111,DROP,"unable to fork: ");
@@ -299,11 +281,6 @@ void doit(int t) {
         }
       }
       
-      if (verbosity >= 2) {
-        strnum[fmt_ulong(strnum,getpid())] = 0;
-        strerr_warn3("sslserver: ssl ",strnum," accept ",0);
-      }
-        
       if (flagclientcert) {
         if (flagclientcert == 2) verifyhost = 0;
         switch(ssl_verify(ssl,verifyhost)) {
@@ -334,6 +311,23 @@ void doit(int t) {
           }
       }
 
+      if (verbosity >= 2) {
+        strnum[fmt_ulong(strnum,getpid())] = 0;
+        if (verbosity >= 3 && ssl_env.len > 1) {
+          s = ssl_env.s;
+          if ((j = str_chr(s,'=')))
+            if (!stralloc_copys(&tlsinfo,s+j+1)) drop_nomem();
+          if (!stralloc_cats(&tlsinfo,":")) drop_nomem();
+          s = s + str_len(s) + 1;
+          s = s + str_len(s) + 1;
+          if ((j = str_chr(s,'=')))
+            if (!stralloc_cats(&tlsinfo,s+j+1)) drop_nomem();
+          if (!stralloc_0(&tlsinfo)) drop_nomem();
+          strerr_warn4("sslserver: ssl ",strnum," accept ",tlsinfo.s,0);
+        } else
+          strerr_warn3("sslserver: ssl ",strnum," accept ",0);
+      }
+
       if (ssl_cmd == 'Y' || ssl_cmd == 'y') {
         if (ssl_io(ssl,pi[1],po[0],progtimeout) != 0) {
           kill(childpid, SIGTERM);
@@ -360,13 +354,13 @@ void doit(int t) {
   if (socket_local6(t,localip,&localport,&scope_id) == -1)
     strerr_die2sys(111,DROP,"unable to get local address: ");
 
+/* Local IP information */
+
   if (mappedv4)
     localipstr[ip4_fmt(localipstr,&localip[12])] = 0;
   else
     localipstr[ip6_compactaddr(localipstr,localip)] = 0;
-
-  remoteportstr[fmt_ulong(remoteportstr,remoteport)] = 0;
-
+    
   if (!localhost)
     if (dns_name6(&localhostsa,localip) == 0)
       if (localhostsa.len) {
@@ -374,6 +368,47 @@ void doit(int t) {
 	localhost = localhostsa.s;
       }
 
+/* Remote IP information */
+
+  stripaddr=remoteipstr;
+  if (!ipv6 && byte_equal(remoteipstr,7,V4MAPPREFIX))
+    stripaddr = remoteipstr+7;
+    
+  remoteportstr[fmt_ulong(remoteportstr,remoteport)] = 0;
+
+  if (flagremotehost)
+    if (dns_name6(&remotehostsa,remoteip) == 0)
+      if (remotehostsa.len) {
+        if (flagparanoid) {
+          verifyhost = remoteipstr;
+          if (!ipv4) {
+            if (dns_ip6(&tmp,&remotehostsa) == 0)
+              for (j = 0;j + 16 <= tmp.len;j += 16)
+                if (byte_equal(remoteip,16,tmp.s + j)) {
+                  flagparanoid = 0;
+                  break;
+                }
+          } else if (dns_ip4(&tmp,&remotehostsa) == 0) {
+              for (j = 0;j + 4 <= tmp.len;j += 4)
+                if (byte_equal(stripaddr,4,tmp.s + j)) {
+                  flagparanoid = 0;
+                  break;
+                }
+          }
+        }
+        if (!flagparanoid) {
+          if (!stralloc_0(&remotehostsa)) drop_nomem();
+            remotehost = remotehostsa.s;
+            verifyhost = remotehostsa.s;
+        }
+      }
+
+  if (flagremoteinfo) {
+    if (remoteinfo6(&tcpremoteinfo,remoteip,remoteport,localip,localport,timeout,netif) == -1)
+      flagremoteinfo = 0;
+    if (!stralloc_0(&tcpremoteinfo)) drop_nomem();
+  }
+
 /* Setup environment variables */
 
   env("PROTO","SSL");
@@ -397,7 +432,7 @@ void doit(int t) {
       env("TCP6REMOTEPORT",remoteportstr);
       env("TCP6REMOTEHOST",remotehost);
     }
-    env("TCPREMOTEIP",remoteipstr);
+    env("TCPREMOTEIP",stripaddr);
     env("TCPREMOTEPORT",remoteportstr);
     env("TCPREMOTEHOST",remotehost);
     if (!mappedv4) {
@@ -409,35 +444,10 @@ void doit(int t) {
     }
   }
 
-  if (flagremotehost)
-    if (dns_name6(&remotehostsa,remoteip) == 0)
-      if (remotehostsa.len) {
-        if (flagparanoid)
-          if (dns_ip6(&tmp,&remotehostsa) == 0)
-            for (j = 0;j + 16 <= tmp.len;j += 16)
-              if (byte_equal(remoteip,16,tmp.s + j)) {
-                flagparanoid = 0;
-                break;
-              }
-        if (!flagparanoid) {
-          if (!stralloc_0(&remotehostsa)) drop_nomem();
-          remotehost = remotehostsa.s;
-        } 
-      }
-
-  if (flagremoteinfo) {
-    if (remoteinfo6(&tcpremoteinfo,remoteip,remoteport,localip,localport,timeout,netif) == -1)
-      flagremoteinfo = 0;
-    if (!stralloc_0(&tcpremoteinfo)) drop_nomem();
-  }
   env("SSLREMOTEINFO",flagremoteinfo ? tcpremoteinfo.s : 0);
   if (flagtcpenv)
     env("TCPREMOTEINFO",flagremoteinfo ? tcpremoteinfo.s : 0);
 
-  stripaddr=remoteipstr;
-  if (!ipv6 && byte_equal(remoteipstr,7,V4MAPPREFIX))
-    stripaddr = remoteipstr+7;
-
 /* Evaluate rules cdb */
 
   if (fnrules) {
@@ -551,7 +561,7 @@ void usage(void)
 {
   strerr_warn1("\
 sslserver: usage: sslserver \
-[ -1346UXpPhHrRoOdDqQvIeEsSnNmzZ ] \
+[ -1346UXpPhHrRoOdDqQvVIeEsSnNmzZ ] \
 [ -c limit ] \
 [ -x rules.cdb ] \
 [ -B banner ] \
@@ -628,7 +638,7 @@ int main(int argc,char * const *argv) {
   int s;
   int t;
  
-  while ((opt = getopt(argc,argv,"1346dDvqQhHrRUXx:t:T:u:g:l:b:B:c:pPoOIEeSsw:nNzZm")) != opteof)
+  while ((opt = getopt(argc,argv,"1346dDvVqQhHrRUXx:t:T:u:g:l:b:B:c:pPoOIEeSsw:nNzZm")) != opteof)
     switch(opt) {
       case 'b': scan_ulong(optarg,&backlog); break;
       case 'c': scan_ulong(optarg,&limit); break;
@@ -638,6 +648,7 @@ int main(int argc,char * const *argv) {
       case 'd': flagdelay = 1; break;
       case 'D': flagdelay = 0; break;
       case 'v': verbosity = 2; break;
+      case 'V': verbosity = 3; break;
       case 'q': verbosity = 0; break;
       case 'Q': verbosity = 1; break;
       case 'P': flagparanoid = 0; break;
@@ -695,35 +706,35 @@ int main(int argc,char * const *argv) {
     uint16_unpack_big((char *)&se->s_port,&localport);
   }
 
-  if (x = env_get("VERIFYDEPTH")) {
+  if ((x = env_get("VERIFYDEPTH"))) {
     scan_ulong(x,&u);
     verifydepth = u;
   }
 
-  if (x = env_get("CAFILE")) cafile = x;
+  if ((x = env_get("CAFILE"))) cafile = x;
   if (cafile && str_equal(cafile,"")) cafile = 0;
 
-  if (x = env_get("CCAFILE")) ccafile = x;
+  if ((x = env_get("CCAFILE"))) ccafile = x;
   if (ccafile && str_equal(ccafile,"")) ccafile = 0;
   if (ccafile && str_equal(ccafile,"-")) flagclientcert = 0;
   if (!flagclientcert) ccafile = 0;
 
-  if (x = env_get("CADIR")) cadir = x;
+  if ((x = env_get("CADIR"))) cadir = x;
   if (cadir && str_equal(cadir,"")) cadir= 0;
 
-  if (x = env_get("CERTCHAINFILE")) certchainfile = x;
+  if ((x = env_get("CERTCHAINFILE"))) certchainfile = x;
   if (certchainfile && str_equal(certchainfile,"")) certchainfile = 0;
 
-  if (x = env_get("CERTFILE")) certfile = x;
+  if ((x = env_get("CERTFILE"))) certfile = x;
   if (certfile && str_equal(certfile,"")) certfile = 0;
 
-  if (x = env_get("KEYFILE")) keyfile = x;
+  if ((x = env_get("KEYFILE"))) keyfile = x;
   if (keyfile && str_equal(keyfile,"")) keyfile = 0;
 
-  if (x = env_get("DHFILE")) dhfile = x;
+  if ((x = env_get("DHFILE"))) dhfile = x;
   if (dhfile && str_equal(dhfile,"")) dhfile = 0;
 
-  if (x = env_get("CIPHERS")) ciphers = x;
+  if ((x = env_get("CIPHERS"))) ciphers = x;
   if (ciphers && str_equal(ciphers,"")) ciphers = 0;
 
   sig_block(sig_child);
@@ -796,8 +807,12 @@ int main(int argc,char * const *argv) {
   if (!ssl_cca(ctx,ccafile))
     strerr_die2x(111,FATAL,"unable to load client CA list");
 
-  if (!ssl_params(ctx,dhfile,rsalen))
-    strerr_die2x(111,FATAL,"unable to set DH/RSA parameters");
+  if (!ssl_params_rsa(ctx,rsalen))
+    strerr_die2x(111,FATAL,"unable to set RSA parameters");
+  if (!ssl_params_dh(ctx,dhfile))
+    strerr_die2x(111,FATAL,"unable to set DH parameters");
+  if (!ssl_params_ecdh(ctx))
+    strerr_warn1("unable to set ECDH parameters",0);
 
   if (!ssl_ciphers(ctx,ciphers))
     strerr_die2x(111,FATAL,"unable to set cipher list");
